/*
* This file is part of Firestorm NIDS
* Copyright (c) 2001 Gianni Tedesco
*
*    Firestorm is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    Firestorm is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with Firestorm; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include <capdev_pcap.h>
#include <s_util.h>

PLUGIN_DECLARE_STD_IMPORTS;
proc_add_captype add_captype;

void *lpf_init(char *interface, char *filter)
{
	p_cpcap_priv priv;
	char buf[4096];
	char *err=(char *)&buf;
	
	if ( !(priv=my_malloc(sizeof(cpcap_priv_t))) ) return NULL;
	memset(priv, 0, sizeof(cpcap_priv_t));

	priv->pcap_desc=pcap_open_offline(interface, err);

	if (!priv->pcap_desc) {
		print_raw(err);
		print_raw(CRLF);
		my_free(priv);
		return NULL;
	}
	
	priv->link = pcap_datalink(priv->pcap_desc);
	
	return priv;
}

int lpf_end(void *priv)
{
	p_cpcap_priv p=(p_cpcap_priv)priv;

	if ( p->pcap_desc ) 
		pcap_close(p->pcap_desc);

	my_free(p);
	
	return 1;
}

void lpf_callback(u_char *user, struct pcap_pkthdr *header, u_char *data)
{
	p_cpcap_priv p;
	uint real_len;

	if ( !(p=(p_cpcap_priv)user) ) return;
	
	real_len=p->pkt.caplen=header->caplen;
	p->pkt.len=header->len;

	p->pkt.layer[0].id = p->link;
	
	if ( header->caplen > CAP_SNAPLEN ) {
		real_len = CAP_SNAPLEN;
	}
	
	p->pkt.data=data;
}

struct packet *lpf_poll(void *priv)
{
	p_cpcap_priv p=NULL;

	if ( !(p=(p_cpcap_priv)priv) ) return NULL;
	
	if ( !pcap_dispatch(p->pcap_desc, 1, 
		(pcap_handler)lpf_callback, priv) ) {
		return NULL;
	}else{
		return &p->pkt;
	}
}

uint _PLUGIN_INIT (p_plugin_in in, p_plugin_out out)
{
	PLUGIN_CHECK_PARAMS(in);
	PLUGIN_IMPORT_STD;
	
	add_captype=in->add_captype;
	
	PLUGIN_NAME("capture/libpcap_pfile");
	PLUGIN_DESC("Capture packets from a file using libpcap");
	PLUGIN_AUTHOR("Gianni Tedesco", "gianni@ecsc.co.uk");
	PLUGIN_LICENSE("GPL");
	PLUGIN_VERS(1,0);
	
	if ( add_captype("libpcap_pfile", lpf_init, lpf_end, lpf_poll) ) {
		return PLUGIN_ERR_OK;
	}else{
		return PLUGIN_ERR_FAIL;
	}
}

/* This is where we would cleanup our shit */
uint _PLUGIN_UNLOAD (void)
{
	return PLUGIN_ERR_OK;
}
