/*
* This file is part of Firestorm NIDS
* Copyright (c) 2001 Gianni Tedesco
*
*    Firestorm is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    Firestorm is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with Firestorm; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#ifndef __CAPDEV_HEADER_INCLUDED__
#define __CAPDEV_HEADER_INCLUDED__

#include <g_globals.h>
#include <lists.h>

enum {
	CAP_STATE_NULL=0,
	CAP_STATE_ENDED=0,
	CAP_STATE_INIT,
	CAP_STATE_CAPTURE
}cap_state;

enum {
	CAP_METHOD_NORMAL,
	CAP_METHOD_POLLME,
}cap_method;

typedef void *(*proc_capdev_init)(char *, char *);
typedef int (*proc_capdev_end)(void *);
typedef struct packet *(*proc_capdev_poll)(void *);

typedef struct captype {
	struct captype		*next;
	char			*name;
	proc_capdev_init	init;
	proc_capdev_end		end;
	proc_capdev_poll	poll;
}captype_t, *p_captype;

typedef struct capdev{
	struct capdev	*next;
	p_captype	capture;
	
	uint		state;
	char		*interface;
	char		*filter;

	void		*priv;
}capdev_t, *p_capdev;

extern p_capdev capdevs;
int capdev_init (void);

void capdev_init_all(void);
void capdev_end_all(int, void *);

p_captype capdev_find_type(char *name);
int capdev_add(char *type, char *interface, char *filter);
int capdev_add_type(char *name, proc_capdev_init i,
		proc_capdev_end e,
	       	proc_capdev_poll p);

/* Given to captype plugins */
typedef int (*proc_add_captype)(char *, proc_capdev_init,
		proc_capdev_end,
		proc_capdev_poll);

#endif /* __LOADER_HEADER_INCLUDED__ */

