/*
* This file is part of Firestorm NIDS
* Copyright (c) 2001 Gianni Tedesco
*
*    Firestorm is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    Firestorm is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with Firestorm; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#ifndef __S_GLOBALS_HEADER_INCLUDED__
#define __S_GLOBALS_HEADER_INCLUDED__

#include <g_globals.h>
#include <packet.h>

typedef enum req_e {
	MATCH_NONE,
	MATCH_GT,
	MATCH_LT,
	MATCH_GTE,
	MATCH_LTE,
	MATCH_EQ,
	MATCH_NEQ
}req_t;

/* List of strings */
typedef struct config_arg {
	list_head_t     l;
	char		*arg;
}config_arg_t, *p_config_arg;

/* A key/value list element */
typedef struct kv {
	list_head_t     l;
	int             flags;
	char            *key;
	char            *val;
}kv_t, *p_kv;

/* Same as above, except 3 strings  */
typedef struct c_capdev{
	list_head_t     l;
	char            *type;
	char            *source;
	char            *filter;
}c_capdev_t, *p_c_capdev;

typedef struct require{
	list_head_t	l;
	req_t		match;
	char		*pname;
	uint		minor,major;
}require_t, *p_require;

typedef struct sensor_globals_t{
	/* Memory management */
	proc_malloc		malloc;
	proc_free		free;
	proc_fs_strdup 		fs_strdup;
	
	/* Global locking and management type stuff */
	char			*config_fn; /* Config file filename */

	/* Count the number of each type of resource allocated */
	uint			num_plugins; /* Number of loaded plugins */
	uint			num_rules; /* Number of rules */
	uint			num_ops; /* Number of match operations */
	uint			num_matches;
	uint			num_rulefiles; /* Rule files loaded */
	uint			num_matchers; /* Matchers */
	uint			num_targets; /* Loaded targets */
	uint			num_captypes; /* Loaded capture types */
	
	/* KILLED AFTER config_go() !! */
	/* ------------------------------- */
	p_config_arg		plugin_dirs; /* List of plugin dirs  */
	p_c_capdev		capdevs;
	p_kv			rulesets;
	p_require		requires; /* Plugin requires */
	char			*root_dir;
	/* ------------------------------- */
	
	/* Security information */
	uid_t			uid;
	gid_t			gid;
	int			chroot;

	/* List of global plugin variables */
	p_kv			plugin_vars;

	struct timeval		start_time;
	struct timeval		end_time;
}sensor_globals_t;

extern struct sensor_globals_t globals;

#endif //__S_GLOBALS_HEADER_INCLUDED__
