/*
* This file is part of Firestorm NIDS
* Copyright (c) 2001 Gianni Tedesco
*
*    Firestorm is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    Firestorm is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with Firestorm; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include <capdev.h>
#include <cleanup.h>
#include <s_globals.h>
#include <s_util.h>

p_captype captype=NULL;
p_capdev capdevs=NULL;

int capdev_init (void)
{
	return 1;
}

int capdev_add(char *type, char *interface, char *filter)
{
	p_captype t=NULL;
	p_capdev  d=globals.malloc(sizeof(capdev_t));
	
	if ( !d ) return 0;

	if ( (t=capdev_find_type(type)) == NULL ) {
		print_out(PRINT_ERROR "Cannot find captype \"%s\"" CRLF, type);
		globals.free(d);
		return 0;
	}

	d->capture=t;
	d->state=CAP_STATE_NULL;
	d->filter=filter;
	d->priv=NULL; /* not set yet */
	
	if ( strcasecmp(interface, "any") ) {
		d->interface=globals.fs_strdup(interface);
	}else{
		d->interface= NULL;
	}
	
	d->next=capdevs;
	capdevs=d;
	
	return 1;
}

void capdev_init_all(void)
{
	p_capdev dev;

	for (dev=capdevs; dev; dev=dev->next) {
		if ( dev->state  != CAP_STATE_NULL ) continue;
		dev->priv = dev->capture->init(dev->interface, dev->filter);

		if ( dev->priv ) {
			dev->state = CAP_STATE_INIT;
		}else{
			dev->state = CAP_STATE_NULL;
		}
	}	
}

void capdev_end_all(int code, void *priv)
{
	p_captype tmp=NULL;
	p_captype foo=NULL;

	p_capdev dev=NULL;
	p_capdev bar=NULL;

	print_raw("Stopping all capture devices" CRLF);
	
	/* Remove all capdevs */
	for (dev=capdevs; dev; dev=dev->next) {
		if ( dev->state == CAP_STATE_NULL ) continue;
		dev->capture->end(dev->priv);
		dev->state = CAP_STATE_NULL;
	}
	for ( dev=capdevs; dev;) {
		bar=dev;
		dev=dev->next;
		if (bar->interface)globals.free(bar->interface);
		if (bar->filter)globals.free(bar->filter);
		globals.free(bar);
		bar=NULL;
	}
	
	/* Remove all captypes */
	for (tmp=captype; tmp;) {
		foo=tmp;
		tmp=tmp->next;
		globals.free(foo);
		foo=NULL;
	}
}

p_captype capdev_find_type(char *name)
{
	p_captype tmp=NULL;
	if ( name == NULL ) return NULL;
	for (tmp=captype; tmp; tmp=tmp->next) {
		if ( strcmp(name, tmp->name) == 0 ) {
			return tmp;
		}
	}
	return NULL;
}

int capdev_add_type( char *name, proc_capdev_init i, 
       proc_capdev_end e, proc_capdev_poll p)
{
	p_captype type=NULL;
	
	if ( !name || !i || !e || !p ) return 0;
	
	if ( !(type=globals.malloc(sizeof(captype_t))) ) return 0;

	type->name=name;
	type->init=i;
	type->end=e;
	type->poll=p;
	
	/* Add the captype */
	globals.num_captypes++;
	type->next=captype;
	captype=type;

	return 1;
}
