/*
* This file is part of Firestorm NIDS
* Copyright (c) 2001 Gianni Tedesco
*
*    Firestorm is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    Firestorm is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with Firestorm; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include <demux.h>
#include <s_util.h>
#include <cleanup.h>

p_demux_layer	dlist[LTYPE_MAX];

/* go through our linked list of demux handlers
 * and try to demultiplex this packet */
int inline demux_layer_from_plugins(int layer, int type, p_packet p)
{
	p_demux_layer d=NULL;
	int t;
 
	for (d=dlist[type]; d; d=(p_demux_layer)d->l.next )
	{
		switch ( (t=d->demux(p, layer)) ) {
			case DEMUX_UNKNOWN:
				break;
			default:
				/* Could use a move-to-front heuristic */
				return t;
		}
	}

	return DEMUX_UNKNOWN;
}

int demux ( p_packet p )
{
	int retval = 0;

	p->layer[0].h.raw = p->data;
	p->layer[0].type=LTYPE_LINK;

	for (retval=0; retval<MAX_DEMUX_LAYERS-1; retval++) {
		switch ( demux_layer_from_plugins(retval, p->layer[retval].type, p)) {
			case DEMUX_UNKNOWN:
				/*
				print_out("Layer %i type (%i) not supported."
					CRLF, retval, p->layer[retval].id);
				*/
			case DEMUX_FINISHED:
			case LTYPE_APP:
				return retval;

			case DEMUX_OK:
				if ( (unsigned long)p->layer[retval+1].h.raw > 
					((unsigned long)p->data + CAP_SNAPLEN)){
					print_out(PRINT_WARNING
						"Demultiplexer found packet to be truncated" CRLF);
				}
				if ( retval == MAX_DEMUX_LAYERS-1 ) {
					print_out(PRINT_WARNING
						"Packet contained more than %i layers!!" CRLF, demux+1);
				}
				break;
		}
	}
	
	return 0;
}

/* Cleanup our lists */
void demux_cleanup(int code, void *priv)
{
	p_demux_layer tmp=NULL, foo=NULL;
	int i;
	
	print_raw("Cleaning up demux() handlers" CRLF);
	for (i=0; i<LTYPE_MAX; i++) {
		for (tmp=dlist[i]; tmp;) {
			foo=tmp;
			tmp=(p_demux_layer)tmp->l.next;
			globals.free(foo);
			foo=NULL;
		}
	}
	return;
}

int demux_init(void)
{
	int i;

	for (i=0; i<LTYPE_MAX; i++){
		dlist[i]=NULL;
	}

	cleanup_add_handler(demux_cleanup, NULL);

	return 1;
}

uint demux_add_handler (uint layer, proc_demux target)
{
	p_demux_layer tmp=NULL;
	
	if ( layer > LTYPE_MAX ) {
		print_out(PRINT_ERROR "DEMUX: plugin tried to add layer "
			"%i handler (?!)" CRLF, layer);
		return 0;
	}

	if ( !(tmp=globals.malloc(sizeof(demux_layer_t))) ) {
		print_raw(PRINT_ERROR "DEMUX: Ran OOM allocating "
			"demux handler" CRLF);
		return 0;
	}

	/* Actually add it */
	tmp->demux = target;
	dlist[layer]=list_append(tmp, dlist[layer]);

	return 1;
}
