/*
* This file is part of Firestorm NIDS
* Copyright (c) 2001 Gianni Tedesco
*
*    Firestorm is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    Firestorm is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with Firestorm; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include <s_globals.h>
#include <rules.h>
#include <cleanup.h>

p_parser parsers=NULL;

int rules_loadfile(char *type, char *fn)
{
	p_parser l;

	if ( !type || !fn ) return 0;

	for (l=parsers; l; l=(p_parser)l->l.next) {
		if ( strcasecmp(l->name, type) == 0 ) {
			if ( l->loadfile(fn) ){
				globals.num_rulefiles++;
				return 1;
			}else{
				return 0;
			}
		}
	}

	return 0;
}

void rules_cleanup(int code, void *priv)
{
	p_parser foo,bar;

	print_raw("Removing parsers" CRLF);	
	if ( parsers == NULL ) return;

	for (foo=parsers; foo; )
	{
		bar=foo;
		foo=(p_parser)foo->l.next;
		globals.free(bar);
		bar=NULL;
	}
	
	return;
}

int rules_init(void)
{
	
	cleanup_add_handler(rules_cleanup, NULL);
	return 1;
}

int rules_add_parser(char *name, proc_parser_loadfile lf)
{
	p_parser p=NULL;

	if ( !name || !lf ) return 0;

	if ( !(p=globals.malloc(sizeof(parser_t))) ) return 0;

	p->name=name;
	p->loadfile=lf;
	
	parsers=list_append(p, parsers);
	
	return 1;
}
